from importlib import import_module
import logging
import simplejson as json


LOGGER = logging.getLogger(__name__)


class Handler:

    # this function runs in a different process...
    def __init__(self, config):
        self.config = config.copy()
        self.handlers = dict()

    # ...than this one, so no conexion can be shared between the two
    def __call__(self, channel, method, properties, body: bytes):
        parsed_event = self.parse_event(body)

        if parsed_event is None:
            return

        self.get_medium('Email').publish(parsed_event)

    def get_medium(self, name):
        if name not in self.handlers:
            module = import_module(
                'charpe.mediums.{}_handler'.format(name.lower()),
            )
            self.handlers[name] = getattr(module, name+'Handler')(self.config)

        return self.handlers[name]

    def parse_event(self, payload):
        try:
            data = json.loads(payload)
        except json.decoder.JSONDecodeError as e:
            LOGGER.warning('Couldn\'t decode event\'s JSON data:')
            return

        return data



from datetime import datetime
from pytz import timezone
import pytz

ISO_FORMAT = '%Y-%m-%dT%H:%M:%SZ'
ES_MX_FORMAT = '%d/%m/%y %H:%M'
tz = timezone('America/Mexico_City')


def datetimeformat(date):
    utc_dt = pytz.utc.localize(datetime.strptime(date, ISO_FORMAT))

    return utc_dt.astimezone(tz).strftime(ES_MX_FORMAT)


def diffinhours(date, otherdate):
    delta = datetime.strptime(date, ISO_FORMAT) - \
            datetime.strptime(otherdate, ISO_FORMAT)
    hours = delta.total_seconds() / 3600

    return '{:5.3}h'.format(hours)



from multiprocessing import Pool
import signal
import pika
import logging

from charpe.handler import Handler

LOGGER = logging.getLogger(__name__)


class Loop:

    def __init__(self, config):
        self.config = config
        self.handler = Handler(config)

        LOGGER.info('Initialized loop')

    def start(self):
        connection = pika.BlockingConnection(pika.ConnectionParameters(
            host=self.config['RABBIT_HOST'],
        ))
        channel = connection.channel()

        channel.exchange_declare(
            exchange=self.config['RABBIT_NOTIFY_EXCHANGE'],
            exchange_type='direct'
        )

        result = channel.queue_declare(exclusive=True)
        queue_name = result.method.queue

        channel.queue_bind(
            exchange=self.config['RABBIT_NOTIFY_EXCHANGE'],
            queue=queue_name,
            routing_key='email',
        )

        channel.basic_consume(
            self.handler,
            queue=queue_name,
            consumer_tag=self.config['RABBIT_CONSUMER_TAG'],
            no_ack=True
        )

        try:
            channel.start_consuming()
        except KeyboardInterrupt:
            LOGGER.info('PVM stopped')



import logging

WORKERS = 2

# Rabbitmq
RABBIT_HOST = 'localhost'
RABBIT_NOTIFY_EXCHANGE = 'cacahuate_notify'
RABBIT_CONSUMER_TAG = 'cacahuate_notifier_consumer_1'
RABBIT_NO_ACK = True

# Log configuration
LOG_LEVEL = logging.INFO

# Time settings
TIMEZONE = 'UTC'

# Needed to send mails
MAIL_SERVER = 'smtp.mailgun.org'
MAIL_PORT = '587'
MAIL_USE_TLS = True
MAIL_USE_SSL = False
MAIL_USERNAME = ''
MAIL_PASSWORD = ''
MAIL_DEFAULT_SENDER = 'procesos@tracsa.com.mx'
MAIL_SUPPRESS_SEND = False

# For template rendering
URL_PROTOCOL = 'https'
URL_SUBDOMAIN = 'com'

# For the telegram handler
TELEGRAM_BOT_KEY = '<bot key>'



from charpe.loop import Loop
from itacate import Config
import time
import os


def main():
    # Load the config
    config = Config(os.path.dirname(os.path.realpath(__file__)))
    config.from_pyfile('settings.py')
    config.from_envvar('CHARPE_SETTINGS', silent=False)

    # Set the timezone
    os.environ['TZ'] = config['TIMEZONE']
    time.tzset()

    # Logging stuff
    import logging

    logging.basicConfig(
        format='[%(levelname)s] %(message)s - %(name)s:%(lineno)s',
        level=config['LOG_LEVEL'],
    )

    # Run the event loop
    loop = Loop(config)
    loop.start()


if __name__ == '__main__':
    main()
